"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var DataUtils_1 = require("./DataUtils");
exports.treeGroupMark = {};
exports.treeDataMark = {};
exports.getExpandedParents = function (treeData, rowKeyField) {
    return treeData
        .filter(function (item) { return item.treeGroupMark === exports.treeGroupMark; })
        .map(function (item) { return DataUtils_1.getValueByField(item.rowData, rowKeyField); });
};
var getItemStructure = function (item, dataHash, rowKeyField, treeDeep) {
    if (treeDeep === void 0) { treeDeep = 0; }
    var children = dataHash[DataUtils_1.getValueByField(item, rowKeyField)];
    if (!children) {
        return [{ treeDataMark: exports.treeDataMark, rowData: item, treeDeep: treeDeep + 1 }];
    }
    var result = [{ treeGroupMark: exports.treeGroupMark, rowData: item, treeDeep: treeDeep }];
    children.forEach(function (c) {
        var childrenData = getItemStructure(c, dataHash, rowKeyField, treeDeep + 1);
        result.push.apply(result, childrenData);
    });
    return result;
};
exports.getTreeGroupChain = function (keyValue, dataMap, treeGroupKeyField, treeGroupKeyValues, groupChain) {
    if (groupChain === void 0) { groupChain = []; }
    var value = dataMap[keyValue];
    var chain = groupChain;
    if (!treeGroupKeyValues.includes(keyValue)) {
        treeGroupKeyValues.push(keyValue);
        chain = __spreadArrays(groupChain, [value]);
    }
    var treeGroupKeyValue = DataUtils_1.getValueByField(value, treeGroupKeyField);
    if (treeGroupKeyValue) {
        return exports.getTreeGroupChain(treeGroupKeyValue, dataMap, treeGroupKeyField, treeGroupKeyValues, chain);
    }
    return chain;
};
exports.restoreFilteredData = function (_a) {
    var data = _a.data, originalData = _a.originalData, rowKeyField = _a.rowKeyField, treeGroupKeyField = _a.treeGroupKeyField, treeGroupsExpanded = _a.treeGroupsExpanded;
    var filteredData = [];
    var treeGroupKeyValues = data.map(function (d) { return DataUtils_1.getValueByField(d, rowKeyField); });
    var dataMap = originalData.reduce(function (acc, d) {
        acc[DataUtils_1.getValueByField(d, rowKeyField)] = d;
        return acc;
    }, {});
    data.forEach(function (d) {
        var treeGroupKeyValue = DataUtils_1.getValueByField(d, treeGroupKeyField);
        if (treeGroupKeyValues.includes(treeGroupKeyValue) || !treeGroupKeyValue) {
            filteredData.push(d);
        }
        else {
            var groupsChain = exports.getTreeGroupChain(treeGroupKeyValue, dataMap, treeGroupKeyField, treeGroupKeyValues);
            filteredData = __spreadArrays(filteredData, groupsChain, [d]);
        }
    });
    return filteredData;
};
var getDataHashAndRootElements = function (_a) {
    var data = _a.data, treeGroupKeyField = _a.treeGroupKeyField, treeGroupsExpanded = _a.treeGroupsExpanded;
    var dataHash = {};
    var rootElements = [];
    data.forEach(function (d) {
        var _a;
        var parentRowKeyValue = (_a = DataUtils_1.getValueByField(d, treeGroupKeyField), (_a !== null && _a !== void 0 ? _a : undefined));
        if (!parentRowKeyValue) {
            rootElements.push(d);
            return;
        }
        if (!dataHash[parentRowKeyValue]) {
            dataHash[parentRowKeyValue] = [];
        }
        if (!treeGroupsExpanded || treeGroupsExpanded.includes(parentRowKeyValue)) {
            dataHash[parentRowKeyValue].push(d);
        }
    });
    return { dataHash: dataHash, rootElements: rootElements };
};
exports.getTreeData = function (_a) {
    var data = _a.data, originalData = _a.originalData, rowKeyField = _a.rowKeyField, treeGroupKeyField = _a.treeGroupKeyField, treeGroupsExpanded = _a.treeGroupsExpanded;
    if (data.length !== originalData.length) {
        data = exports.restoreFilteredData({
            data: data,
            originalData: originalData,
            rowKeyField: rowKeyField,
            treeGroupKeyField: treeGroupKeyField,
            treeGroupsExpanded: treeGroupsExpanded
        });
    }
    var _b = getDataHashAndRootElements({
        data: data,
        treeGroupKeyField: treeGroupKeyField,
        treeGroupsExpanded: treeGroupsExpanded
    }), dataHash = _b.dataHash, rootElements = _b.rootElements;
    var newData = [];
    rootElements.forEach(function (d) {
        newData.push.apply(newData, getItemStructure(d, dataHash, rowKeyField));
    });
    return newData;
};
